  unit Com_4;

  interface

  Uses Crt;

  type
    PortType	= (COM1, COM2, COM3, COM4);
    BaudType	= (B110, B150, B300, B600, B1200, B2400, B4800,
  		   B9600, B19200, B38400, B57600, B115200);
    ParityType	= (None, Odd, Even, Mark, Space);
    LengthType	= (D5, D6, D7, D8);
    StopType	= (S1, S2);
    FlowType	= (No, RtsCts, XonXoff);

  procedure InitCom (PortNumber : PortType;
  		     BaudRate : BaudType;
                     ParityBit : ParityType;
  		     DataLength : LengthType;
  		     StopBits : StopType;
  		     FlowControl : FlowType);
  procedure ExitCom (PortNumber : PortType);
  function  ComReceived (PortNumber : PortType) : boolean;
  function  ReadCom (PortNumber : PortType) : char;
  function  ComAllowed (PortNumber : PortType) : boolean;
  procedure WriteCom (PortNumber : PortType; OutByte : char);
  procedure BreakCom (PortNumber : PortType);

  implementation

  type
    INS8250 = record
      DLL : integer;  { divisor latch low register (if LCR bit7 = 1) }
      DLH : integer;  { divisor latch high register (if LCR bit7 = 1) }
      THR : integer;  { transmit holding register }
      RHR : integer;  { receive holding register }
      IER : integer;  { interrupt enable register }
      LCR : integer;  { line control register }
      MCR : integer;  { modem control register }
      LSR : integer;  { line status register }
      MSR : integer;  { modem status register }
      Opened : boolean;
      FC     : FlowType;
      Flow   : Boolean;
    end;
Var
  ComPort : array [PortType] of INS8250;

procedure InitCom (PortNumber : PortType;
		   BaudRate : BaudType;
                   ParityBit : ParityType;
  		   DataLength : LengthType;
  		   StopBits : StopType;
  		   FlowControl : FlowType);
Const
  Ptab : Array[ParityType] Of Byte = ($00, $08, $18, $28, $38);
Begin
  With ComPort[PortNumber] Do Begin
    If RHR = 0 Then Exit;
    Port[LCR] := $80;
    Port[DLH] := 0;
    Case BaudRate Of
      B110    : Begin Port[DLH] := $04; Port[DLL] := $17; End;
      B150    : Begin Port[DLH] := $03; Port[DLL] := $00; End;
      B300    : Begin Port[DLH] := $01; Port[DLL] := $80; End;
      B600    : Begin Port[DLH] := $00; Port[DLL] := $C0; End;
      B1200   : Begin Port[DLH] := $00; Port[DLL] := $60; End;
      B2400   : Begin Port[DLH] := $00; Port[DLL] := $30; End;
      B4800   : Begin Port[DLH] := $00; Port[DLL] := $18; End;
      B9600   : Begin Port[DLH] := $00; Port[DLL] := $0C; End;
      B19200  : Begin Port[DLH] := $00; Port[DLL] := $06; End;
      B38400  : Begin Port[DLH] := $00; Port[DLL] := $03; End;
      B57600  : Begin Port[DLH] := $00; Port[DLL] := $02; End;
      B115200 : Begin Port[DLH] := $00; Port[DLL] := $01; End;
      End;
    Port[LCR] := Ptab[ParityBit] Or (Ord(StopBits) Shl 2) Or Ord(DataLength);
    Port[MCR] :=$03;
    Opened := True;
    FC := FlowControl;
    If FC = RtsCts Then Flow := (Port[MSR] and $10) <> 0
                   Else Flow := True;
    End;
  End;

Procedure ExitCom (PortNumber : PortType);
Begin
  With ComPort[PortNumber] Do Begin
    If Not Opened Then Exit;
    Port[MCR] := 0;
    Opened := False;
    End;
  End;

Function  ComReceived (PortNumber : PortType) : boolean;
Begin
  With ComPort[PortNumber] Do Begin
    ComReceived := Opened And ((Port[LSR] And $01) <> 0);
    End;
  End;

Function  ReadCom (PortNumber : PortType) : char;
Var
  C : Char;
Begin
  With ComPort[PortNumber] Do Begin
    If Not Opened Then Exit;
    C := Char(Port[RHR]);
    If FC = XonXoff Then Case C of
      ^S : Flow := False;
      ^Q : Flow := True;
      End;
    ReadCom := C;
    End;
  End;

Function  ComAllowed (PortNumber : PortType) : boolean;
Begin
  With ComPort[PortNumber] Do Begin
    ComAllowed := False;
    if Not Opened Then Exit;
    if (Port[LSR] And $20) = 0 Then Exit;
    Case FC of
      RtsCts  : ComAllowed := (Port[MSR] and $10) <> 0;
      XonXoff : ComAllowed := Flow;
      No      : ComAllowed := True;
      End;
    End;
  End;

Procedure WriteCom (PortNumber : PortType; OutByte : char);
Begin
  With ComPort[PortNumber] Do Begin
    If Not Opened Then Exit;
    Port[THR] := Byte(OutByte);
    End;
  End;

Procedure BreakCom (PortNumber : PortType);
Var
  OldLCR : Byte;
Begin
  With ComPort[PortNumber] Do Begin
    If Not Opened Then Exit;
    oldLCR := Port[LCR];
    Port[LCR] := OldLCR Or $40;
    Delay(500);
    Port[LCR] := OldLCR;
    End;
  End;


Var
  port : PortType;
  base : Word;

Begin
  for port := Com1 to Com4 Do With ComPort[port] Do Begin
    Opened := false;
    Base := MemW[$40:Ord(Port)*2];
    DLL := Base+0;
    DLH := Base+1;
    THR := Base+0;
    RHR := Base+0;
    IER := Base+1;
    LCR := Base+3;
    MCR := Base+4;
    LSR := Base+5;
    MSR := Base+6;
    End;
  End.
